/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.ToolbarSeparator");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.rtc.foundation.web.ui.util.Flags");
dojo.require("com.ibm.team.rtc.foundation.web.ui.util.Sprites");
dojo.require("com.ibm.team.rtc.foundation.web.ui.util.sprite.ImageSource");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.OperationLogConsole");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.template.WorkItemTemplateSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.query.SortCriteria"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var checkNotNull			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty			= net.jazz.ajax.toolbox.lang.checkNotEmpty;

var Toolbar					= jazz.ui.Toolbar;
var Button					= jazz.ui.toolbar.Button;
var TwoStateButton			= jazz.ui.toolbar.TwoStateButton;

var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var WidgetUtil				= com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var WindowContext			= com.ibm.team.workitem.web.ui.internal.module.WindowContext;
var Sprites					= com.ibm.team.rtc.foundation.web.ui.util.Sprites;
var ImageSource				= com.ibm.team.rtc.foundation.web.ui.util.sprite.ImageSource;
var SortCriteria			= com.ibm.team.workitem.web.client.internal.query.SortCriteria;
var WorkItemTemplateSection	= com.ibm.team.workitem.web.ui.internal.view.template.WorkItemTemplateSection;
var EventTopics				= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var Flags					= com.ibm.team.rtc.foundation.web.ui.util.Flags;
/*
 * Constants
 */
var BULK_OPERATIONS_WIDGET_ID	= "queryresult/bulkoperationswidget";
var BUNDLE_ID					= "com.ibm.team.workitem.web";
var PAGE_SIZE_INTERVALS			= [10, 25, 50, 75, 100, 250, 500];

/**
 * A meta-area widget that allows the user to manipulate query results by
 * navigating between query result pages and by performing textual filters on a
 * query page.
 */
dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl",
	[dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.cache.internal.OperationLogConsole],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/templates/QueryResultPageControl.html"),
		textDir: null,
		// populate the template with data, connect events, and show initial sub-tab
		constructor: function() {
			this.messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		}, 
		postCreate: function() {
			checkNotNull(this.query, "this.query"); /* QueryDescriptor */
			checkNotNull(this.queryResultsSummaryPage, "this.queryResultsSummaryPage");
			checkNotNull(this.queryResultsWidget, "this.queryResultsWidget");
			checkNotNull(this.parentView, "this.parentView");
			
			if (this.isBottomControl) {
				this.domNode.className= "queryResultPageControlBottom";
				this.topRow.style.display= "none";
			}

			this._bulkOperationResultDiv= document.createElement("div");
			var noResultDiv= document.createElement("DIV");
			noResultDiv.style.color= "#CCC";
			noResultDiv.appendChild(document.createTextNode(this.messages.labelNoResults));
			this._bulkOperationResultDiv.appendChild(noResultDiv);

			this._stopOperation();
			var links= this.titleContainer.getElementsByTagName("a");
			if (!this.textDir) {
				this.textDir= WidgetUtil.getWidgetBaseTextDir();
			}
			
			var name= this.query.getName() ? this.query.getName() : this.messages.titleNewQuery;
			name= WorkItemUtil.plainToHTML(name)
			var shortName= (name.length> 55) ? name.substring(0,52) + "..." : name;
			this.titleContainer.innerHTML= shortName;
			this.titleContainer.title =name;
			if (!!this.textDir) {
				this.setBaseTextDir();
				this._onStorageChangeHandle= dojo.subscribe(WidgetUtil.BIDI_STORAGE_CHANGED, this, "setBaseTextDir");
			}					
			
			if (this._getResultCount() > 0) {
				this._showPagingControls();
			}
			this._toolbar= new Toolbar();
			this._simpleToolbarContainer.appendChild(this._toolbar.domNode);
			this.filterControl.setAttribute("aria-label", this.messages.labelFilterText);
			var self= this;			
			var editQueryButton= new Button({
					title: this.messages.titleEditCurrentQuery,
					enabled: (this.query.getItemId() && !this.query.isNew && this.query.isEditable() && window.location.href.indexOf("action=newQuery") == -1), 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/layout/images/edit-current-query.gif")), 
					alt: this.messages.buttonEdit,
					onClick: dojo.hitch(this, function(event) { 
								if (!editQueryButton.enabled) return;
								WindowContext.setWindowLocation2(WorkItemUtil.getEditQueryUri(self.query.getItemId()));
					})
			});

			this._editMultipleButton= new TwoStateButton({
					title: this.messages.titleShowHideBulkOperations,
					enabled: false, // enabled below 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/layout/images/edit-multiple-wi.gif")), 
					alt: this.messages.buttonEditTable,
					onClick: dojo.hitch(this, function(event) {
								dojo.stopEvent(event);
								if (!self._editMultipleButton.enabled) return;
								self._handleTableEditClick(self._editMultipleButton.pressed);
					})
			});
			this._createTemplateButton= new TwoStateButton({
					title: this.messages.buttonCreateTemplate,
					enabled: true, 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/template/images/crttemplate.gif")), 
					alt: this.messages.buttonCreateTemplate,
					onClick: dojo.hitch(this, function(event) {
								dojo.stopEvent(event);
								self._handleCreateWorkItemTemplatesClick(self._createTemplateButton.pressed);
					})
			});

			this._exportButton= new Button({
					title: this.messages.titleExportCSV,
					enabled: (this.query.supportsCsvExport()), 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/query/images/q-result-export-table.gif")),
					alt: this.messages.titleExportCSV,
					onClick: dojo.hitch(this, function(event) {
								dojo.stopEvent(event);
								if (!self._exportButton.enabled) return;
								this._openDocument(self.query.getCsvExportLink(self.queryRun) + "&nocache=" + (new Date()).getTime(), "_blank"); // Open in a new window
					})
			});

			this._setTableEditImageState();
			this._toolbar.addChild(editQueryButton);
			this._toolbar.addChild(this._editMultipleButton);
			this._toolbar.addChild(this._createTemplateButton);
			this._toolbar.addChild(this._exportButton);


			// REFRESH
			if (this.queryUrl) {
				this.refreshUrl= WorkItemUtil.reuniquifyUrl(this.queryUrl);
			} else {
				this.refreshUrl= WorkItemUtil.reuniquifyUrl(WorkItemUtil.getQueryUri(this.query), true);
			}
			
	
			this._refreshButton= new Button({	
					title: this.messages.titleReRunQuery,
					enabled: true, 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/editor/images/refresh.gif")), 
					alt: this.messages.titleReRunQuery,
					onClick: dojo.hitch(this, function(event) {
									dojo.stopEvent(event);
									if (!self._refreshButton.enabled) return;
									self._handleRefreshClick();
					})	
			});
			this._toolbar.addChild(this._refreshButton);	
			this._toolbar.addChild(new dijit.ToolbarSeparator());			

			this._printButton= new Button({	
					title:  this.messages.titleViewFullReport,
					enabled: (this.query.supportsHtmlExport()), // Enabled when export is enabled
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/query/images/q-result-print.gif")), 
					alt:  this.messages.titleViewFullReport,
					onClick: dojo.hitch(this, function(event) {
									dojo.stopEvent(event);
									if (!self._printButton.enabled) return;
									window.open(this.query.getHtmlExportLink(this.queryRun) + "&nocache=" + (new Date()).getTime(), "_blank");
					})
			});	
			this._toolbar.addChild(this._printButton);
			this._resetFilter();
			
			this._bulkOperationsMomento= com.ibm.team.workitem.web.cache.internal.Cache.getItem(BULK_OPERATIONS_WIDGET_ID);
			if ( !this.isBottomControl && this._bulkOperationsMomento) {
				this._operationSection= this._bulkOperationsMomento.operationSection;
				this.setTableEditMode(this._bulkOperationsMomento.isShown);
			}
			
			var helpContextId= com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics.QUERY_RESULTS_PAGE_CS_HELP;
			this._pageHelp= new jazz.app.ua.PageHoverHelp({uaKey: helpContextId});
			this._pageHelp.placeAt(this._contextHelp);
			
		},
		destroy: function() {
			this._stopOperation();
			this._attributeTree= false;
			if (this._toolbar) this._toolbar.destroy();
			if (this._pageHelp) this._pageHelp.destroy();
			this._pageHelp= false;
			this._toolbar= false;
			if (this._onStorageChangeHandle) dojo.unsubscribe(this._onStorageChangeHandle);			
			this.inherited(arguments);
		},
		_openDocument: function(href) {
			if (dojo.isIE && dojo.isIE <= 9 && href.length > 2048) {
				window.alert(this.messages.urlTooLong);
			} else {
  				window.open(href);
			}
		},
		_handleTableEditClick: function(isPressed) {
			if (this._createTemplateButton.pressed) this._createTemplateButton.setAttribute('pressed', false);
			this.logClearMessages();
			// TODO(QueryDescriptor): New concept for indicating bulk edit mode
			this.setTableEditMode(!this.query._isTableEditModeSelected);
		},
		_handleCreateWorkItemTemplatesClick: function(isPressed) {
			if (isPressed) {
				if (this._editMultipleButton.pressed) this._editMultipleButton.setAttribute('pressed', false);
				if ( !this._templateSection) {
					this._templateSection= new WorkItemTemplateSection({
						queryResultPageControl: this,
						projectAreaId: this.query.getProjectArea()
					});
				}
				this._replaceChildren(this._operationViewletArea, this._templateSection.domNode);
				this.bulkOperationRow.style.display= "";
			} else {
				this.bulkOperationRow.style.display= "none";
				if (this._createTemplateButton.pressed) this._createTemplateButton.setAttribute('pressed', false);
			}
			this.getQueryResultsWidget().showCheckboxes(isPressed, this._attributeTree);
		},
		refresh: function() {
			this._handleRefreshClick();
		},
		_handleRefreshClick: function(e) {
			this.getQueryResultsWidget().refreshLastQuery(this.query);
		},
		setRunButtonEnabled: function() {
				if (this._bulkOperationsMomento && this._bulkOperationsMomento.operationSection) {
					this._bulkOperationsMomento.operationSection.setRunButtonEnabled();	
				}
		},
		showBulkOperations: function(isVisible) {
			if (isVisible) {
				this.getQueryResultsWidget()._hideAllEditors();				
			}

			if (!this._bulkOperationsMomento) {
				// no memento found: try and fetch it from the cache
				this._bulkOperationsMomento= com.ibm.team.workitem.web.cache.internal.Cache.getItem(BULK_OPERATIONS_WIDGET_ID);
			} 
			if (this._bulkOperationsMomento) {
				// initialize variables from memento
			    if (this._bulkOperationsMomento.operationSection) {
					 this._bulkOperationsMomento.operationSection.queryResultPageControl= this;
				}
				this._attributeTree= this._bulkOperationsMomento.attributeTree;
				this._bulkContainer= this._bulkOperationsMomento.container;
				this._bulkOperationsMomento.isShown= isVisible;
				this.setRunButtonEnabled();
			}
			
			this.getQueryResultsWidget().showCheckboxes(isVisible, this._attributeTree);
			
			if (this._bulkContainer) {
				// set visibility and return
				this._replaceChildren(this._operationViewletArea, this._bulkContainer.domNode);
				var displayValue= isVisible ? "" : "none";
				this.bulkOperationRow.style.display= displayValue;
				return;
			}
			
			// no memento found: create the bulk operation part
			if (this._hasShownBulkOperations) return; // clicked while loading
			this._hasShownBulkOperations= true; // avoid double clicking on the icon while loading
						
			this._bulkOperationEditorDiv= document.createElement("div");
			var loadingDiv= document.createElement("DIV");
			loadingDiv.style.color= "#CCC";
			loadingDiv.appendChild(document.createTextNode(this.messages.busyLoading));
			this._bulkOperationEditorDiv.appendChild(loadingDiv);
			this._replaceChildren(this._operationViewletArea, this._bulkOperationEditorDiv);

			this.bulkOperationRow.style.display= "";
 			if (!this.isBottomControl) {
				if (!this._bulkContainer) {
					var srh= new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_setAttributeTree", "_handleAttributeError");
					com.ibm.team.workitem.web.client.internal.QueryClient.getAttributeTree(srh);
				}
				else {
					this._replaceChildren(this._bulkOperationEditorDiv, this._bulkContainer.domNode);
				}
			}
		},
		_replaceChildren: function(node, child) {
			while(node.childNodes.length > 0) {
				node.removeChild(node.firstChild);
			}
			node.appendChild(child);
		},
		/*
		 * OperationLog functions
		 */
		logSetVisible: function(isVisible) {
			this.inherited(arguments);
		},
		logClearMessages: function() {
			this.showHeaderMessage(" ", "errorMessageArea-disabled");
			dojox.xml.parser.removeChildren(this._bulkOperationResultDiv);
			this.inherited(arguments);
		},
		logAppendMessageElement: function(messageNode) { 
			this._bulkOperationResultDiv.appendChild(messageNode);
			this.inherited(arguments);
		},
		logStop: function(showTerminatingMessage) {
			this._stopOperation(showTerminatingMessage);
		},
		logSetStopButtonEnabled: function(isEnabled) {
			if (this._bulkContainer) 	this._bulkContainer.setStopButtonEnabled(isEnabled);
		},
		logMessage: function(parms /* message */) {
			console.debug(parms.message);
			this.inherited(arguments);
		},
		logTerminatingMessage: function(parms /* message, details */) {
			this.logStop(/* show Terminating message */ false);
			
			this._currentTerminatingMessage= parms;
			var msg, severity;
			if (parms.severity == 0) {
			    msg= parms.message;
				severity= "errorMessageAreaInfo";
			} else {
				if (parms.details) {
					msg= dojo.string.substitute(this.messages.errorTerminatingStep, [parms.message || "", parms.details ]);									
				} else {
					msg= dojo.string.substitute(this.messages.errorTerminatingStepWithoutDetails, [parms.message || "" ]);				
				}
 				console.debug(msg);
				if (parms.severity == 4) {
					severity= "errorMessageAreaWarning";
				} else {
					severity= "errorMessageArea";
				}
			}
			this.showHeaderMessage(msg, severity);			
		},
		/*
		 * Busy messages
		 */
		_stopOperation: function(showTerminatingMessage) {
			dojo.publish(EventTopics.STOP_BACKGROUND_OPERATIONS, [ showTerminatingMessage]);
		},
		showHeaderMessage: function (message, severityClassName) {
			dojo.removeClass(this._errorDiv);
			dojo.addClass(this._errorDiv, severityClassName);

			var messageNode;

			if (typeof message === "string") {
				messageNode= document.createElement("a");
				messageNode.appendChild(document.createTextNode(message));
				messageNode.setAttribute("href", "#");
				dojo.connect(messageNode, "onclick", this._handleErrorMessageClick);
			} else  if (this._isDomNode(message)) {
				messageNode= message;
			}
			this._replaceChildren(this._errorDiv, messageNode);
	  	},
	  	_isDomNode: function(o) {
	  		return typeof HTMLElement === "object" ? o instanceof HTMLElement :
	  			typeof o === "object" && typeof o.nodeType === "number";
	  	},
		_handleAttributeError: function(error) {
			alert(error.message);
		},
		addBulkOperationAttribute: function( attributeId) {
			if (this._bulkContainer) {
				this._bulkContainer.sectionWidget.addBulkOperationAttribute(attributeId);
			}
		},
		_setAttributeTree: function(tree) {
			this._attributeTree= tree;
			this._createBulkOperationPart(this._bulkOperationEditorDiv);
		},
		_createBulkOperationPart: function(node) {
			if (this._bulkContainer) {
//95375			com.ibm.team.workitem.web.cache.internal.Cache.decrementUseCount(BULK_OPERATIONS_WIDGET_ID);
// decrement use count will do a destroy this._bulkContainer.destroy();
				this._bulkContainer= null;
			}
			
			var opSection= new com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection({
				stopButton: this.stopButton,
				queryResultPageControl: this,
				attributes: this._attributeTree,
				projectAreaItemId: this.query.getProjectArea()
			});
			var container= new com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer({
				sectionWidget: opSection
			});
			this._operationSection= opSection;
			this._replaceChildren(node, container.domNode);
			
			this._bulkContainer= container;
			this._bulkOperationsMomento= {
				operationSection: this._operationSection,
				attributeTree: this._attributeTree,
				container: container,
				isShown: true,
				destroy: function() {
					this.container ? this.container.destroy() : null;
				}
			} 
			com.ibm.team.workitem.web.cache.internal.Cache.addItem(BULK_OPERATIONS_WIDGET_ID, this._bulkOperationsMomento);
			com.ibm.team.workitem.web.cache.internal.Cache.incrementUseCount(BULK_OPERATIONS_WIDGET_ID);

		},

		setTableEditMode: function(isTableEditModeSelected) {
			// TODO(QueryDescriptor): New concept for indicating bulk edit mode
			this.query._isTableEditModeSelected= isTableEditModeSelected == true;
			/*
			 * Save all open work items before going into edit mode
			 */
			// TODO(QueryDescriptor): New concept for indicating bulk edit mode
			if (this.query._isTableEditModeSelected) {
				 var operationManager= new com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation({
					log: this
				});
				operationManager.saveOpenWorkItems({
					ignoreIsExecuting: true,
					onSaveAllSuccess: dojo.hitch(this, this.saveAllWorkItemsSuccess),
					onSaveAllError: dojo.hitch(this,this.saveAllWorkItemsError)
					});
			} else {
				this._setTableEditImageState();
				// TODO(QueryDescriptor): New concept for indicating bulk edit mode
				this.showBulkOperations(this.query._isTableEditModeSelected);
			}
		},
		
		saveAllWorkItemsError: function() {
			this.setTableEditMode(false);
		},
		saveAllWorkItemsSuccess: function() {
			   
			this._setTableEditImageState();	
		
			// Show bulk operations window
			// TODO(QueryDescriptor): New concept for indicating bulk edit mode
			this.showBulkOperations(this.query._isTableEditModeSelected);	
		
		},
		_setTableEditImageState: function() {		
			// TODO: Disable when no results
			this._editMultipleButton.setAttribute('enabled', true);
		},
		_createLink: function(url, style, tooltip) {
			var link= document.createElement("a");
			link.title= tooltip;
			dojo.addClass(link, style);
			if (url) {
				link.href= url;
			} else {
				dojo.addClass(link, "disabledIcon");
			}
			return link;
		},
		handleRowSelectionChanged: function() {
			if (this._operationSection) this._operationSection.handleRowSelectionChanged();
			if (this._templateSection) this._templateSection.handleRowSelectionChanged();
			
		},
		_createSeparator: function() {
			var div= document.createElement("div");
			dojo.addClass(div, "separator");
			return div;
		},

		//
		// event handlers related to the filter control
		//
		getQueryResultsWidget: function() {
			return this.parentView._getQueryResultsWidget();
		}, 
		_onSetFilterValue: function(e) {
				this.getQueryResultsWidget().filter(this.filterControl.value);
		},
		
		_onFilterFocus: function(e) {
			if (this.filterControl.__jazzIsReset) {
				this.filterControl.value= "";
				dojo.removeClass(this.filterControlWrapper, "disabledText");
			}
			this.filterControl.__jazzIsReset= false;
		},
		
		_onFilterBlur: function(e) {
			var filterVal= dojo.trim(this.filterControl.value);
			if (filterVal.length === 0) {
				this._resetFilter();
			} else {
				this.filterControl.value= filterVal;
			}
		},
		setFilter: function(filterVal) {
				this.filterControl.value= filterVal;
				dojo.removeClass(this.filterControlWrapper, "disabledText");
				this.filterControl.__jazzIsReset= false;
				if (filterVal == "") {
					this._resetFilter();
				}
		},		
		_onClearFilterClick: function(e) {
			dojo.stopEvent(e);
			this._resetFilter();
		},
			
		_resetFilterControl: function() {
			if (this.filterControl.value =="") {
				this.getQueryResultsWidget().filter("");
				dojo.addClass(this.filterControlWrapper,"disabledText");
				this.filterControl.value= this.messages.labelTypeFilterText;
				this.filterControl.__jazzIsReset= true;
			}
		},	
		_resetFilter: function() {
			// unfilter results
			this.getQueryResultsWidget().filter("");
			dojo.addClass(this.filterControlWrapper,"disabledText");
			this.filterControl.value= this.messages.labelTypeFilterText;
			this.filterControl.__jazzIsReset= true;
		},
		
		//
		// logic for displaying the paging controls
		//
		
		_showPagingControls: function() {
			
			// show items per page drop-down
			if (!this.isBottomControl) {
				this.itemsPerPageDiv.appendChild(this._getItemsPerPageElement());
			}
			// show previous page text or link
			this.pageNumberDiv.appendChild(this._getPreviousElement());
			
			// show count info
			var pageInfoSpan= document.createElement("span");
			
			pageInfoSpan.className= "pageInfoSpan";
			var endIndex= Math.min(this._getMaxPageIndex(), this._getResultCount());
			var actual= this._getActualCountInfo();
			var resultCount= this._getResultCount() + "";
			if (endIndex == 1) {
				if (actual) {
					pageInfoSpan.innerHTML= dojo.string.substitute(this.messages.labelPage1OfXWithActual, 
						[ resultCount, (actual + "") ]);
				} else {
					pageInfoSpan.innerHTML= dojo.string.substitute(this.messages.labelPage1OfX, 
						[ resultCount ]);
				}
			} else {
				var startIndex= this._getStartIndex() + "";
				var pageIndex= Math.min(this._getMaxPageIndex(), resultCount)  + "";
				if (actual) {
					pageInfoSpan.innerHTML= dojo.string.substitute(this.messages.labelPageXToYOfZWithActual, 
						[ startIndex, pageIndex, resultCount, (actual+"") ]);
				} else {
					pageInfoSpan.innerHTML= dojo.string.substitute(this.messages.labelPageXToYOfZ, 
						[ startIndex, pageIndex, resultCount ]);
				}
			}
			this.pageNumberDiv.appendChild(pageInfoSpan);
			
			// show next page link or text
			this.pageNumberDiv.appendChild(this._getNextElement());
			
			// filter only appears at top
			if (this.isBottomControl) {
				this.filterControlWrapper.style.display= "none";
			}
		},
		
		_getActualCountInfo: function() {
			var count= this._getResultCount();
			var actual= this._getTotal();
			if (count < actual) {
				return actual;
			}
			return null;
		},
		
		/*
		 * Creates and returns the dom node of the page size drop down with
		 * label div.
		 */
		_getItemsPerPageElement: function() {
			var div= document.createElement("div");
			div.className= "itemsPerPage";
			
			var select= document.createElement("select");
			select.title= this.messages.labelItemsPerPage;
			var count= this._getResultCount();
			
			// populate the drop-down
			var optionIndex= 0;
			var defaultSize= WorkItemUtil._getDefaultQueryPageSize();
			var currentSize= this.queryItemsPerPage;
			if (!currentSize) {
				currentSize= count > defaultSize ? defaultSize : "all";
			}
			for (var i= 0; i < PAGE_SIZE_INTERVALS.length; i++) {
				var pageSize= PAGE_SIZE_INTERVALS[i];
				if (count > pageSize) {
					var option= new Option("" + pageSize, "" + pageSize);
					select.options[optionIndex++]= option;
					option.selected= (pageSize == currentSize);
				}
				else break;
			}
			// show all not supported for large queries
			if (count <= PAGE_SIZE_INTERVALS[PAGE_SIZE_INTERVALS.length - 1]) {
				var option= new Option(this.messages.labelShowAll, "all");
				select.options[optionIndex]= option;
				option.selected= (currentSize == "all");
			}
			
			// disable drop-down if only one choice
			if (count <= PAGE_SIZE_INTERVALS[0]) {
				select.disabled= true;
				dojo.addClass(select, "disabled");
			}
			
			// register event handler
			var self= this;
			select.onchange= dojo.hitch(this, function() {
				var itemsPerPage= select.options[select.selectedIndex].value;
				
				// only include sorting information if it is custom
				var orderByAttribute, orderDirection;
				if (this.queryRun && this.queryRun.getUserSortCriteria()) {
					var sortCriteria= this.queryRun.getUserSortCriteria();
					orderByAttribute= sortCriteria.getAttributeId();
					orderDirection= sortCriteria.isAscending() ? SortCriteria.ASCENDING : SortCriteria.DESCENDING;
				}
				
				if (self.parentView && self.parentView.controller && self.parentView.controller.pageController) {
					self.parentView.controller.pageController.setWindowLocationHref(WorkItemUtil.getQueryUri(this.query, 1, orderByAttribute, orderDirection, itemsPerPage, true));
				} else {
					location.href= WorkItemUtil.getQueryUri(this.query, 1, orderByAttribute, orderDirection, itemsPerPage, true);
				}
			});
			
			div.appendChild(select);
			var labelSpan= document.createElement("span");
			labelSpan.className= "itemsPerPageText";
			labelSpan.innerHTML= this.messages.labelItemsPerPage;
			div.appendChild(labelSpan);
			return div;
		},
		
		_getPreviousElement: function() {
			if (this._getStartIndex() > 1) {
				var orderByAttribute, orderDirection;
				if (this.queryRun && this.queryRun.getUserSortCriteria()) {
					var sortCriteria= this.queryRun.getUserSortCriteria();
					orderByAttribute= sortCriteria.getAttributeId();
					orderDirection= sortCriteria.isAscending() ? SortCriteria.ASCENDING : SortCriteria.DESCENDING;
				}
				return this._getLinkElement(WorkItemUtil.getQueryUri(this.query, Math.max(this._getStartIndex() - this.getPageSize()), orderByAttribute, orderDirection, this.queryItemsPerPage, true), this.messages.labelPrevious);
			} else {
				return this._getDisabledTextElement(this.messages.labelPrevious);
			}
		},
		
		_getNextElement: function() {
			if (this._getMaxPageIndex() < this._getResultCount()) {
				var orderByAttribute, orderDirection;
				if (this.queryRun && this.queryRun.getUserSortCriteria()) {
					var sortCriteria= this.queryRun.getUserSortCriteria();
					orderByAttribute= sortCriteria.getAttributeId();
					orderDirection= sortCriteria.isAscending() ? SortCriteria.ASCENDING : SortCriteria.DESCENDING;
				}
				return this._getLinkElement(WorkItemUtil.getQueryUri(this.query, this._getMaxPageIndex() + 1,  orderByAttribute, orderDirection, this.queryItemsPerPage, true), this.messages.labelNext);
			} else {
				return this._getDisabledTextElement(this.messages.labelNext);
			}
		},
		
		_getDisabledTextElement: function(str) {
			var span= document.createElement("span");
			span.className= "disabledText";
			span.appendChild(document.createTextNode(str));
			return span;
		},
		
		_getLinkElement: function(href, text) {
			checkNotEmpty(text, "text");
			checkNotEmpty(href, "href");
			
			var linkElement= document.createElement("a");
			linkElement.href= href;
			linkElement.appendChild(document.createTextNode(text));
			
			return linkElement;
		},
		
		_getStartIndex: function() {
			checkNotNull(this.queryResultsSummaryPage.startIndex, "this.queryResultsSummaryPage.startIndex");
			
			return Number(this.queryResultsSummaryPage.startIndex) + 1;
		},
		
		_getResultCount: function() {
			checkNotNull(this.queryResultsSummaryPage.limit, "this.queryResultsSummaryPage.limit");
			
			return Number(this.queryResultsSummaryPage.limit);
		},
		
		_getTotal: function() {
			var total= new Number(this.queryResultsSummaryPage.totalCount);
			var estimatedTotal= new Number(this.queryResultsSummaryPage.estimatedTotal);
			
			if (estimatedTotal > total)
				return estimatedTotal;
			
			return total;
		},
		
		_getMaxPageIndex: function() {
			if (this.queryItemsPerPage == "all") {
				return this._getResultCount();
			}
			return this._getStartIndex() + this.getPageSize() - 1;
		},
		
		getPageSize: function() {
			if (this.queryItemsPerPage && this.queryItemsPerPage != "all") {
				return Number(this.queryItemsPerPage);
			}
			return WorkItemUtil._getDefaultQueryPageSize();
		},
		_handleErrorMessageClick: function(e) {
			dojo.stopEvent(e);
			if (this._currentTerminatingMessage) {
				var msg= this._currentTerminatingMessage.details;
				if (msg) 
					alert(msg);
			}
		},
		
		setBaseTextDir: function(dir) {
			if (!!dir) {
				this.textDir= dir;
			}
			var defDir= dojo.style(dojo.body(),"direction");
			var baseTextDir= this.textDir;
			var text= this.titleContainer.title.replace(/[\u202a\u202b\u202c]/g,"");
			if (this.textDir == "auto") {
				baseTextDir= WidgetUtil.getStringBaseTextDir(text,defDir);
			}
			if (this.textDir == "none") {
				dojo.style(this.titleContainer,"direction","");
				dojo.style(this.titleContainer,"unicodeBidi","");
				this.titleContainer.title= text;
			}
			else{
				dojo.style(this.titleContainer,"direction",baseTextDir);
				dojo.style(this.titleContainer,"unicodeBidi","embed");
				this.titleContainer.title= WidgetUtil.setSegmentBaseTextDir(text,baseTextDir);
			}
		}
	});

})(); 