/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.core.loader");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.Presentation");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.VariableControl");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorLabel");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ParameterValueWidget");

dojo.require("com.ibm.team.workitem.web.client.internal.query.AttributeExpression");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Operator");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Variable");
dojo.require("com.ibm.team.workitem.web.client.internal.query.UIItem");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;
var VariableControl= com.ibm.team.workitem.web.ui.internal.view.queryeditor.VariableControl;
var loader= jazz.core.loader;
var AttributeExpression= com.ibm.team.workitem.web.client.internal.query.AttributeExpression;
var Operator= com.ibm.team.workitem.web.client.internal.query.Operator;
var Variable= com.ibm.team.workitem.web.client.internal.query.Variable;
var UIItem= com.ibm.team.workitem.web.client.internal.query.UIItem;
var ErrorLabel= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorLabel;
var Presentation=com.ibm.team.workitem.web.ui.internal.view.queryeditor.Presentation;
var ParameterValueWidget= com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ParameterValueWidget;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/ConditionSection.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textLoading = this.messages.busyLoading;
		this.attributeId= args.attributeId; // attribute identifier
		this.parent= args.parent; // TermSection
		this.expression= args.expression; // AttributeExpression
		this.projectAreaItemId= (args.parent) ? args.parent.projectAreaItemId : null; // String
		this.hasParameterValue= false;
		
		this._variableControls= new Array();
		this._operator= null;
	},
	
	getProjectAreaItemId: function() {
		return this.projectAreaItemId;
	},
	
	postCreate: function() {
		this._errorLabel= new ErrorLabel();
		this._errorDiv.appendChild(this._errorLabel.domNode);
		this._changeHandle= dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.QUERY_EDITOR_CHANGED, dojo.hitch(this, this._handleConditionChanged));
		this._changeHandle2= dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.QUERY_CONDITION_SECTION_CHANGED, dojo.hitch(this, this._handleConditionChanged));
	},
	destroy: function() {
			try {
				if (this._errorLabel) this._errorLabel.destroy();
				if (this._conditionWidget && this._conditionWidget.destroy) this._conditionWidget.destroy();
				if (this._parameterValueWidget && this._parameterValueWidget.destroy) this._parameterValueWidget.destroy();
				if (this._changeHandle) dojo.unsubscribe(this._changeHandle);
				if (this._changeHandle2) dojo.unsubscribe(this._changeHandle2);
				
				dojo.forEach(this._variableControls, function(control) {
					if (control.destroy) control.destroy();
				});
				
				this.inherited(arguments);
			} catch (e) {
				
			}
	},
	setSectionSite: function(site) {
		this._site= site;
		this._loadPresentation();
	},
	
	/* AttributeExpression */ getExpression: function() {
		/* Variable[] */ var variables;
		/* UIItem[] */ var values;
		if (this.hasParameterValue) {
			variables= this._presentationWidget.getParameterVariables();
			values= [];
		} else {
			variables= this._getVariables();
			values= this._getValues();
		}
		
		if (values.length > 0 || variables.length > 0 || (this._operator && this._operator.getArity() == 0)) {
			var expression= new AttributeExpression(this.attributeId, this._operator);
			expression.setVariables(variables);
			expression.setValues(values);
			return expression;
		}
		return null;
	},
	clearCondition: function() {
		this._site.onClose();
	},
	onClose: function() {
		this.parent.deregister(this);
		return true;
	},
	
	setParameterValue: function(hasParameterValue) {
		if (this.hasParameterValue === hasParameterValue) {
			return;
		}
		if (hasParameterValue) {
			if (this.isEditorPart()) {
				this._columns.style.display= "none";
				this._filters.style.display= "none";
			}
			this._conditionContent.replaceChild(this._parameterValueWidget.domNode, this._conditionWidget.domNode);
			this._presentationWidget= this._parameterValueWidget;
		} else {
			if (this.isEditorPart() && (this._variableControls.length > 0 || this._filters.hasChildNodes())) {
				this._columns.style.display= "block";
				this._filters.style.display= "block";
			}
			this._conditionContent.replaceChild(this._conditionWidget.domNode, this._parameterValueWidget.domNode);
			this._presentationWidget= this._conditionWidget;
		}
		this.hasParameterValue= hasParameterValue;
	},
	
	isEditorPart: function() {
		return this._site && this._site.isEditorPart();
	},
	
	addToolbar: function(toolbar) {
		this._filters.style.display= "block";
		this._columns.style.display= "block";
		this._filters.appendChild(toolbar.domNode);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_loadPresentation: function() {
		var srh= new ServiceResponseHandler(this, "_handlePresentation", "_handlePresentationError");
		QueryClient.getPresentation(srh, this.attributeId, this.projectAreaItemId);
	},
	
	_handlePresentation: function(/* PresentationDTO */ presentation) {
		this._createWidget(Presentation.createFromDTO(presentation));
	},
	
	_createWidget: function(/* Presentation */ presentation) {
		
		if (!presentation) {
			this._hideLoadingMessage();
			return;
		}
			
		var site= this._site;
		var operator= this.expression ? this.expression.getOperator() : presentation.getOperators()[0];
		this._setOperator(operator);
		
		site.setTitle(presentation.getAttribute().getLabel());
		site.setupMenu("label", dojo.hitch(this, this._handleSetOperator));
		site.setMenuItems(Operator.createDTOs(presentation.getOperators()));
		site.setSelectedMenuItem(operator.createDTO());

		var /* Variable[] */ nonParameterVariables= [];
		if (presentation.getVariables()) {
			for(var i= 0; i < presentation.getVariables().length; i++) {
				if (presentation.getVariables()[i].getId() !== Variable.PARAMETER_VARIABLE_ID) {
					var curVar= presentation.getVariables()[i];
					nonParameterVariables.push(curVar);
				}
			}
		}
		this._createVariableControls(nonParameterVariables, this.expression ? this.expression.getVariables(this.isEditorPart()) : []);
		this._loadWidget(presentation);
		
		if (nonParameterVariables.length > 0 && !this.hasParameterValue) {
			this._columns.style.display= "block";
			this._filters.style.display= "block";
		}
	},
	_loadWidget: function(/* Presentation */ presentation) {
		
		var widgetClass= presentation.getWidgetClass();
		
		if (dojo.exists(widgetClass)) {
			this._showWidget(presentation);
			return;
		}
		
		var self= this;
		loader.load_async(widgetClass, function() {
			try {
				self._showWidget(presentation)
			}
			catch (e) {
				self._handleWidgetError(e);
			}
		});
	},
	
	_showWidget: function(/* Presentation */ presentation) {
		
		this._hideLoadingMessage();
		
		var /* UIItemDTO[] */ selectedValues= [];
		var /* VariableDTO[] */ selectedVariables= [];
		if (this.expression) {
			selectedValues= UIItem.createDTOs(this.expression.getValues());
			selectedVariables= Variable.createDTOs(this.expression.getVariables(this.isEditorPart()));
		}
		var errorCallback= dojo.hitch(this, this._showError);
		
		var ctor= dojo.getObject(presentation.getWidgetClass());
		var args = { presentation: presentation, selection: selectedValues, variables: selectedVariables, errorCallback: errorCallback, section: this };
		if (this.parent && this.parent.queryResultPageControl) {
			args.pageControl = this.parent.queryResultPageControl;
		}
		if (this.projectAreaItemId) {
			args.projectAreaItemId = this.projectAreaItemId;
		}
		var widget= new ctor(args);
		widget.container = this;
		this._conditionContent.appendChild(widget.domNode);
		this._conditionWidget= widget;
		this._parameterValueWidget= new ParameterValueWidget(args);
		this._presentationWidget= this._conditionWidget;

		if (this._hasConditionValues()) {
			this.setParameterValue(typeof selectedVariables !== "undefined" && selectedVariables.length > 0 && selectedVariables[0].id === Variable.PARAMETER_VARIABLE_ID);
		}
		this._site.setToggleParameterAction(this._hasConditionValues());
		this._handleConditionChanged();
	},
	setDerivedValueTitle: function(parms /* presentation, id */) {
		var title = dojo.string.substitute(this.messages.sectionTitleBulkOperationsDerivedValue, [parms.presentation.getAttribute().getLabel(), parms.id]);
		this._site.setTitle(title);
	},
	/* boolean */ _hasConditionValues: function() {
		return this._conditionWidget.presentation.getWidgetClass() !== "com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.MessageWidget";
	},
	
	_createVariableControls: function(/* Variable[] */ variables, /* Variable[] */ used) {
		
		if (!variables) {
			variables= [];
		}
		if (!used) {
			used= [];
		}
		if (variables.length == 0 && !this._filters.hasChildNodes() && this.isEditorPart()) {
			this._columns.style.display= "none";
			return;
		}
		
		var seen= {};
		var k= 0;
		
		for (var i= 0; i < variables.length; i++) {
			this._createVariableCheckBox(variables[i].createDTO(), k % 2);
			seen[variables[i].getLabel()]= true;
			k++;
		}
		for (var i= 0; i < used.length; i++) {
			if (!seen[used[i].getLabel()] && !(Variable.PARAMETER_VARIABLE_ID === used[i].getId())) {
				this._createVariableCheckBox(used[i].createDTO(), k % 2);
				k++;
			}
		}
	},
	
	_createVariableCheckBox: function(/* VariableDTO */ variable, column) {
		var variableControl= new VariableControl({ variable: variable, section: this });
		var columnDiv= this["_column" + column];
		columnDiv.appendChild(variableControl.domNode);
		this._variableControls.push(variableControl);
		
		if (this._isSelected(variable)) {
			variableControl.setChecked(true);
		}
	},
	
	_isSelected: function(/* VariableDTO */ variable) {
		if (this.expression && this.expression.getVariables(false)) {
			var /* Variable[] */ selected= this.expression.getVariables(false);
			for (var i= 0; i < selected.length; i++) {
				if (selected[i].getId() == variable.id && selected[i].getArguments() == variable.arguments)
					return true;
			}
		}
		return false;
	},
	
	/* Variable[] */ _getVariables: function() {
		var variables= new Array();
		
		var controls= this._variableControls;
		if (controls) {
			for (var i= 0; i < controls.length; i++) {
				if (controls[i].isChecked())
					variables.push(controls[i].getVariable());
			}
		}
		var widgetVariables= this._getWidgetVariables();
		if (widgetVariables) {
			for (var i= 0; i < widgetVariables.length; i++) {
				variables.push(widgetVariables[i]);
			}
		}
			
		return variables;
	},
	
	/* Variable[] */ _getWidgetVariables: function() {
		var widget= this._presentationWidget;
		if (!widget || !widget.getVariables)
			return [];
		return widget.getVariables();
	},
	
	/* UIItem[] */ _getValues: function() {
		var widget= this._presentationWidget;
		if (!widget) {
			return [];
		}
		/* UIItemDTO[] */ var values= widget.getValues();
		/* UIItem[] */ var result= [];
		for ( var idx= 0; idx < values.length; idx++) {
			result.push(UIItem.createFromDTO(values[idx]));
		}
		return result;
	},
	
	_handleConditionChanged: function() {
		if (this._conditionWidget && this._conditionWidget.updateStatusLabel && this._presentationWidget && this._presentationWidget._select) {
			this._conditionWidget.updateStatusLabel(this._operator.getLabel(), this._getValues(), this._getVariables());
		}
	},
	
	_handleSetOperator: function(/* OperatorDTO */ operator) {
		this._setOperator(Operator.createFromDTO(operator));
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_setOperator: function(/* Operator */ operator) {
		this._operator= operator;
	},
	
	_hideLoadingMessage: function() {
		dojox.xml.parser.removeChildren(this._loadingMessage);
		this._loadingMessage.style.display = "none";
	},
	
	_handlePresentationError: function(error) {
		this._showError(this.messages.errorFetchingPresentation, error);
	},
	
	_handleWidgetError: function(error) {
		this._showError(this.messages.errorFetchingWidget, error);
	},
	
	_showError: function(message, error) {
		dojox.xml.parser.replaceChildren(this._loadingMessage, document.createTextNode(this.messages.errorFetchingWidget));
		this._loadingMessage.style.color = "red";
		this._loadingMessage.style.display = "";
		this._errorDiv.style.display = '';
		this._errorLabel.showError(message, error);
	}
	
});
	
})();
